/*
 *
 * check_cpu.c - detect CPU has SSE/MMX/3DNow!
 * by Osamu Kayasono <jacobi@jcom.home.ne.jp>
 * Mar 3, 2001
 */

#include "check_cpu.h"

/* cpuid, from kernel source ( linux/include/asm-i386/processor.h ) */
inline void cpuid(int op, int *eax, int *ebx, int *ecx, int *edx)
{
	__asm__("cpuid"
		: "=a" (*eax),
		  "=b" (*ebx),
		  "=c" (*ecx),
		  "=d" (*edx)
		: "a" (op));
}

int check_cpu(void)
{
	int j[5];

	/* check CPU has CPUID */
	cpuid(0,&j[1],&j[2],&j[3],&j[4]);
	if ( j[1] < 1) return 0;

	/* check CPU has SSE */
	cpuid(1,&j[1],&j[2],&j[3],&j[4]);
	if ( (j[4] & FLAG_SSE ) == FLAG_SSE) return HAS_SSE;

	/* check CPU has 3DNow! */
	cpuid(0x80000001,&j[1],&j[2],&j[3],&j[4]);
	if ( (j[4] & FLAG_3DNOW ) == FLAG_3DNOW) return HAS_3DNOW;

	/* check CPU has MMX */
	cpuid(1,&j[1],&j[2],&j[3],&j[4]);
	if ( (j[4] & FLAG_MMX ) == FLAG_MMX) return HAS_MMX;

	return 0;
}

